# !/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Filename: conftest.py
# Project: tests
# Author: Brian Cherinka
# Created: Thursday, 14th February 2019 6:04:55 pm
# License: BSD 3-clause "New" or "Revised" License
# Copyright (c) 2019 Brian Cherinka
# Last Modified: Sunday, 3rd March 2019 4:53:45 pm
# Modified By: Brian Cherinka


from __future__ import print_function, division, absolute_import

# import some fixtures we might need
from .models import model_a_factory, model_b_factory
from .database import session


